/*
openIRC
Copyright (C) 2006  target0 (target0@geeknode.org)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/


#ifndef _OPENIRC_H
#define _OPENIRC_H

#define HOSTLEN     64
#define NICKLEN     32
#define CHANNELLEN  50
#define SENDQSIZE   65536
#define CHUNKSIZE   65536

typedef struct {
    char nick[NICKLEN+1];
    char ident[NICKLEN+1];
    char host[HOSTLEN+1];
} ircUser;

typedef struct {
    char target[CHANNELLEN+1];
    char data[1024+1];
} ircMsgData;

#define IRC_MSG_RAW     0
#define IRC_MSG_PRIV    1
#define IRC_MSG_NOTICE  2
#define IRC_MSG_QUIT    3
#define IRC_MSG_KICK    4
#define IRC_MSG_INVITE  5
#define IRC_MSG_MODE    6
#define IRC_MSG_JOIN    7
#define IRC_MSG_PART    8
#define IRC_MSG_TOPIC   9

typedef struct __ircCallback {
    void (*func)();
    int id;
    char command[64];
    struct __ircCallback *next, *prev;
} ircCallback;

typedef struct {
    int sock;
    char hostname[HOSTLEN+1];
    unsigned short port;
    unsigned int timeout;

    char nick[NICKLEN+1];
    char ident[NICKLEN+1];
    char realname[NICKLEN+1];

    struct {
        char chunkbuf[CHUNKSIZE];
        char *chunkbufentry;
        char *nextline, *currentline;
        struct pollfd pfd;
    } indata;

    struct {
        char outbuf[SENDQSIZE];
        int wbytes;
        struct pollfd pfd;
    } outdata;

    ircCallback *callbacks;
} ircClient;

#define ERR_SUCCESS     0
#define ERR_FAILED      1
#define ERR_TIMEOUT     2
#define ERR_MAXSENDQ    3
#define ERR_NODATA      4

#define IRC_LOOP(x) while ((ircLoop((x))) != ERR_FAILED);

#define ircsprintf(a,b,c,d) va_start(d,c); \
                            vsnprintf(a,b,c,d); \
                            va_end(d)

ircClient *ircGetClient (void);
void ircFreeClient (ircClient *);
int ircConnect (ircClient *);
int ircSend (ircClient *, char *, ...);
void ircHandShake (ircClient *);
void ircDisconnect (ircClient *);
int ircRead (ircClient *);
int ircGetLine (ircClient *);
char *SeperateWord (char *);
char *StripBlanks (char *);
int ircLoop (ircClient *);
ircCallback *ircAddCallback (ircClient *, int, char *, void (*func)());
ircCallback *ircFindCallback (ircClient *, char *);
void ircRemoveCallback (ircClient *, ircCallback *);
void ircParse (ircClient *);
ircUser *ircParseUser (char *);
ircMsgData *ircParseData (char *);
int ircSendPriv (ircClient *, char *, char *, ...);
int ircSendNotice (ircClient *, char *, char *, ...);
int ircSendQuit (ircClient *, char *, ...);
int ircSendJoin (ircClient *, char *);
int ircSendPart (ircClient *, char *, char *, ...);
int ircSendTopic (ircClient *, char *, char *, ...);
int ircSendMode (ircClient *, char *, char *, ...);
int ircSendKick (ircClient *, char *, char *, char *, ...);

#endif
